@@ -37,17 +37,17 @@ pub(crate) type VersionID = u64;
 /// Avoid making mistakes between branch name and version name.
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub struct BranchName<'a>(pub &'a [u8]);
-#[derive(Clone, Debug, Eq, PartialEq)]
+#[derive(Clone, Debug, Eq, PartialEq, Default)]
 pub struct BranchNameOwned(pub Vec<u8>);
 /// Avoid making mistakes between branch name and version name.
-#[derive(Clone, Copy, Debug, Eq, PartialEq)]
+#[derive(Clone, Copy, Debug, Eq, PartialEq, Default)]
 pub struct ParentBranchName<'a>(pub &'a [u8]);
-#[derive(Clone, Debug, Eq, PartialEq)]
+#[derive(Clone, Debug, Eq, PartialEq, Default)]
 pub struct ParentBranchNameOwned(pub Vec<u8>);
 /// Avoid making mistakes between branch name and version name.
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub struct VersionName<'a>(pub &'a [u8]);
-#[derive(Clone, Debug, Eq, PartialEq)]
+#[derive(Clone, Debug, Eq, PartialEq, Default)]
 pub struct VersionNameOwned(pub Vec<u8>);
 
 pub const KB: u64 = 1 << 10;
