@@ -4,8 +4,9 @@
 
 use crate::{
     common::ende::{KeyEnDe, ValueEnDe},
-    versioned::mapx_ord_rawkey::{MapxOrdRkVs, MapxOrdRkVsIter},
-    BranchName, ParentBranchName, VersionName, VsMgmt,
+    versioned::mapx_ord_rawkey::{MapxOrdRawKeyVs, MapxOrdRawKeyVsIter},
+    BranchName, BranchNameOwned, ParentBranchName, VersionName, VersionNameOwned,
+    VsMgmt,
 };
 use ruc::*;
 use serde::{Deserialize, Serialize};
@@ -18,7 +19,7 @@ use std::{
 #[derive(Clone, Serialize, Deserialize, PartialEq, Eq, Debug)]
 #[serde(bound = "")]
 pub struct MapxVs<K, V> {
-    inner: MapxOrdRkVs<V>,
+    inner: MapxOrdRawKeyVs<V>,
     p: PhantomData<K>,
 }
 
@@ -40,7 +41,7 @@ where
     #[inline(always)]
     pub fn new() -> Self {
         MapxVs {
-            inner: MapxOrdRkVs::new(),
+            inner: MapxOrdRawKeyVs::new(),
             p: PhantomData,
         }
     }
@@ -398,7 +399,7 @@ where
     K: KeyEnDe,
     V: ValueEnDe,
 {
-    iter: MapxOrdRkVsIter<'a, V>,
+    iter: MapxOrdRawKeyVsIter<'a, V>,
     p: PhantomData<K>,
 }
 
