@@ -39,7 +39,7 @@ mod test;
 
 use crate::{
     common::{BranchName, ParentBranchName, RawKey, RawValue, VersionName, NULL},
-    VsMgmt,
+    BranchNameOwned, VersionNameOwned, VsMgmt,
 };
 use ruc::*;
 use serde::{Deserialize, Serialize};
@@ -82,7 +82,7 @@ impl MapxRawVs {
         value: &[u8],
         branch_name: BranchName,
     ) -> Result<Option<RawValue>> {
-        let branch_id = self.inner.get_branch_id(branch_name).c(d!())?;
+        let branch_id = self.inner.branch_get_id_by_name(branch_name).c(d!())?;
         self.inner.insert_by_branch(key, value, branch_id).c(d!())
     }
 
@@ -99,7 +99,7 @@ impl MapxRawVs {
         key: &[u8],
         branch_name: BranchName,
     ) -> Result<Option<RawValue>> {
-        let branch_id = self.inner.get_branch_id(branch_name).c(d!())?;
+        let branch_id = self.inner.branch_get_id_by_name(branch_name).c(d!())?;
         self.inner.remove_by_branch(key, branch_id).c(d!())
     }
 
@@ -126,7 +126,7 @@ impl MapxRawVs {
         key: &[u8],
         branch_name: BranchName,
     ) -> Option<RawValue> {
-        let branch_id = self.inner.get_branch_id(branch_name)?;
+        let branch_id = self.inner.branch_get_id_by_name(branch_name)?;
         self.inner.get_by_branch(key, branch_id)
     }
 
@@ -138,8 +138,8 @@ impl MapxRawVs {
         branch_name: BranchName,
         version_name: VersionName,
     ) -> Option<RawValue> {
-        let branch_id = self.inner.get_branch_id(branch_name)?;
-        let version_id = self.inner.get_version_id(version_name)?;
+        let branch_id = self.inner.branch_get_id_by_name(branch_name)?;
+        let version_id = self.inner.version_get_id_by_name(version_name)?;
         self.inner.get_by_branch_version(key, branch_id, version_id)
     }
 
@@ -160,7 +160,7 @@ impl MapxRawVs {
         key: &[u8],
         branch_name: BranchName,
     ) -> Option<(RawKey, RawValue)> {
-        let branch_id = self.inner.get_branch_id(branch_name)?;
+        let branch_id = self.inner.branch_get_id_by_name(branch_name)?;
         self.inner.get_ge_by_branch(key, branch_id)
     }
 
@@ -174,8 +174,8 @@ impl MapxRawVs {
         branch_name: BranchName,
         version_name: VersionName,
     ) -> Option<(RawKey, RawValue)> {
-        let branch_id = self.inner.get_branch_id(branch_name)?;
-        let version_id = self.inner.get_version_id(version_name)?;
+        let branch_id = self.inner.branch_get_id_by_name(branch_name)?;
+        let version_id = self.inner.version_get_id_by_name(version_name)?;
         self.inner
             .get_ge_by_branch_version(key, branch_id, version_id)
     }
@@ -197,7 +197,7 @@ impl MapxRawVs {
         key: &[u8],
         branch_name: BranchName,
     ) -> Option<(RawKey, RawValue)> {
-        let branch_id = self.inner.get_branch_id(branch_name)?;
+        let branch_id = self.inner.branch_get_id_by_name(branch_name)?;
         self.inner.get_le_by_branch(key, branch_id)
     }
 
@@ -211,8 +211,8 @@ impl MapxRawVs {
         branch_name: BranchName,
         version_name: VersionName,
     ) -> Option<(RawKey, RawValue)> {
-        let branch_id = self.inner.get_branch_id(branch_name)?;
-        let version_id = self.inner.get_version_id(version_name)?;
+        let branch_id = self.inner.branch_get_id_by_name(branch_name)?;
+        let version_id = self.inner.version_get_id_by_name(version_name)?;
         self.inner
             .get_le_by_branch_version(key, branch_id, version_id)
     }
@@ -226,7 +226,10 @@ impl MapxRawVs {
     /// Create an iterator over a specified branch.
     #[inline(always)]
     pub fn iter_by_branch(&self, branch_name: BranchName) -> MapxRawVsIter {
-        let branch_id = self.inner.get_branch_id(branch_name).unwrap_or(NULL);
+        let branch_id = self
+            .inner
+            .branch_get_id_by_name(branch_name)
+            .unwrap_or(NULL);
         self.inner.iter_by_branch(branch_id)
     }
 
@@ -237,8 +240,14 @@ impl MapxRawVs {
         branch_name: BranchName,
         version_name: VersionName,
     ) -> MapxRawVsIter {
-        let branch_id = self.inner.get_branch_id(branch_name).unwrap_or(NULL);
-        let version_id = self.inner.get_version_id(version_name).unwrap_or(NULL);
+        let branch_id = self
+            .inner
+            .branch_get_id_by_name(branch_name)
+            .unwrap_or(NULL);
+        let version_id = self
+            .inner
+            .version_get_id_by_name(version_name)
+            .unwrap_or(NULL);
         self.inner.iter_by_branch_version(branch_id, version_id)
     }
 
@@ -258,7 +267,10 @@ impl MapxRawVs {
         branch_name: BranchName,
         bounds: R,
     ) -> MapxRawVsIter<'a> {
-        let branch_id = self.inner.get_branch_id(branch_name).unwrap_or(NULL);
+        let branch_id = self
+            .inner
+            .branch_get_id_by_name(branch_name)
+            .unwrap_or(NULL);
         self.inner.range_by_branch(branch_id, bounds)
     }
 
@@ -270,8 +282,14 @@ impl MapxRawVs {
         version_name: VersionName,
         bounds: R,
     ) -> MapxRawVsIter<'a> {
-        let branch_id = self.inner.get_branch_id(branch_name).unwrap_or(NULL);
-        let version_id = self.inner.get_version_id(version_name).unwrap_or(NULL);
+        let branch_id = self
+            .inner
+            .branch_get_id_by_name(branch_name)
+            .unwrap_or(NULL);
+        let version_id = self
+            .inner
+            .version_get_id_by_name(version_name)
+            .unwrap_or(NULL);
         self.inner
             .range_by_branch_version(branch_id, version_id, bounds)
     }
@@ -292,7 +310,10 @@ impl MapxRawVs {
         branch_name: BranchName,
         bounds: R,
     ) -> MapxRawVsIter<'a> {
-        let branch_id = self.inner.get_branch_id(branch_name).unwrap_or(NULL);
+        let branch_id = self
+            .inner
+            .branch_get_id_by_name(branch_name)
+            .unwrap_or(NULL);
         self.inner.range_ref_by_branch(branch_id, bounds)
     }
 
@@ -304,8 +325,14 @@ impl MapxRawVs {
         version_name: VersionName,
         bounds: R,
     ) -> MapxRawVsIter<'a> {
-        let branch_id = self.inner.get_branch_id(branch_name).unwrap_or(NULL);
-        let version_id = self.inner.get_version_id(version_name).unwrap_or(NULL);
+        let branch_id = self
+            .inner
+            .branch_get_id_by_name(branch_name)
+            .unwrap_or(NULL);
+        let version_id = self
+            .inner
+            .version_get_id_by_name(version_name)
+            .unwrap_or(NULL);
         self.inner
             .range_ref_by_branch_version(branch_id, version_id, bounds)
     }
@@ -348,7 +375,7 @@ impl MapxRawVs {
     #[inline(always)]
     pub fn len_by_branch(&self, branch_name: BranchName) -> usize {
         self.inner
-            .get_branch_id(branch_name)
+            .branch_get_id_by_name(branch_name)
             .map(|id| self.inner.len_by_branch(id))
             .unwrap_or(0)
     }
@@ -363,10 +390,10 @@ impl MapxRawVs {
         version_name: VersionName,
     ) -> usize {
         self.inner
-            .get_branch_id(branch_name)
+            .branch_get_id_by_name(branch_name)
             .and_then(|br_id| {
                 self.inner
-                    .get_version_id(version_name)
+                    .version_get_id_by_name(version_name)
                     .map(|ver_id| self.inner.len_by_branch_version(br_id, ver_id))
             })
             .unwrap_or(0)
@@ -419,7 +446,7 @@ impl VsMgmt for MapxRawVs {
         branch_name: BranchName,
     ) -> Result<()> {
         self.inner
-            .get_branch_id(branch_name)
+            .branch_get_id_by_name(branch_name)
             .c(d!("branch not found"))
             .and_then(|br_id| {
                 self.inner
@@ -428,11 +455,19 @@ impl VsMgmt for MapxRawVs {
             })
     }
 
+    #[inline(always)]
+    fn version_exists_globally(&self, version_name: VersionName) -> bool {
+        self.inner
+            .version_get_id_by_name(version_name)
+            .map(|verid| self.inner.version_exists_globally(verid))
+            .unwrap_or(false)
+    }
+
     /// Check if a verison exists on default branch.
     #[inline(always)]
     fn version_exists(&self, version_name: VersionName) -> bool {
         self.inner
-            .get_version_id(version_name)
+            .version_get_id_by_name(version_name)
             .map(|id| self.inner.version_exists(id))
             .unwrap_or(false)
     }
@@ -445,10 +480,10 @@ impl VsMgmt for MapxRawVs {
         branch_name: BranchName,
     ) -> bool {
         self.inner
-            .get_branch_id(branch_name)
+            .branch_get_id_by_name(branch_name)
             .and_then(|br_id| {
                 self.inner
-                    .get_version_id(version_name)
+                    .version_get_id_by_name(version_name)
                     .map(|ver_id| self.inner.version_exists_on_branch(ver_id, br_id))
             })
             .unwrap_or(false)
@@ -476,7 +511,7 @@ impl VsMgmt for MapxRawVs {
     #[inline(always)]
     fn version_pop_by_branch(&self, branch_name: BranchName) -> Result<()> {
         self.inner
-            .get_branch_id(branch_name)
+            .branch_get_id_by_name(branch_name)
             .c(d!("branch not found"))
             .and_then(|br_id| self.inner.version_pop_by_branch(br_id).c(d!()))
     }
@@ -491,7 +526,7 @@ impl VsMgmt for MapxRawVs {
     #[inline(always)]
     unsafe fn version_rebase(&self, base_version: VersionName) -> Result<()> {
         self.inner
-            .get_version_id(base_version)
+            .version_get_id_by_name(base_version)
             .c(d!())
             .and_then(|bv| self.inner.version_rebase(bv).c(d!()))
     }
@@ -509,11 +544,53 @@ impl VsMgmt for MapxRawVs {
         base_version: VersionName,
         branch_name: BranchName,
     ) -> Result<()> {
-        let bv = self.inner.get_version_id(base_version).c(d!())?;
-        let brid = self.inner.get_branch_id(branch_name).c(d!())?;
+        let bv = self.inner.version_get_id_by_name(base_version).c(d!())?;
+        let brid = self.inner.branch_get_id_by_name(branch_name).c(d!())?;
         self.inner.version_rebase_by_branch(bv, brid).c(d!())
     }
 
+    #[inline(always)]
+    fn version_list(&self) -> Result<Vec<VersionNameOwned>> {
+        self.inner.version_list().c(d!())
+    }
+
+    #[inline(always)]
+    fn version_list_by_branch(
+        &self,
+        branch_name: BranchName,
+    ) -> Result<Vec<VersionNameOwned>> {
+        self.inner
+            .branch_get_id_by_name(branch_name)
+            .c(d!("branch not found"))
+            .and_then(|brid| self.inner.version_list_by_branch(brid).c(d!()))
+    }
+
+    #[inline(always)]
+    fn version_list_globally(&self) -> Vec<VersionNameOwned> {
+        self.inner.version_list_globally()
+    }
+
+    #[inline(always)]
+    fn version_has_change_set(&self, version_name: VersionName) -> Result<bool> {
+        self.inner
+            .version_get_id_by_name(version_name)
+            .c(d!("version not found"))
+            .and_then(|verid| self.inner.version_has_change_set(verid).c(d!()))
+    }
+
+    #[inline(always)]
+    fn version_clean_up_globally(&self) -> Result<()> {
+        self.inner.version_clean_up_globally().c(d!())
+    }
+
+    #[inline(always)]
+    unsafe fn version_revert_globally(&self, version_name: VersionName) -> Result<()> {
+        self.inner
+            .version_get_id_by_name(version_name)
+            .c(d!("version not found"))
+            .and_then(|verid| self.inner.version_revert_globally(verid).c(d!()))
+    }
+
     /// Create a new branch based on the head of the default branch.
     #[inline(always)]
     fn branch_create(
@@ -535,7 +612,7 @@ impl VsMgmt for MapxRawVs {
         base_branch_name: ParentBranchName,
     ) -> Result<()> {
         self.inner
-            .get_branch_id(BranchName(base_branch_name.0))
+            .branch_get_id_by_name(BranchName(base_branch_name.0))
             .c(d!("base branch not found"))
             .and_then(|base_br_id| {
                 self.inner
@@ -559,11 +636,11 @@ impl VsMgmt for MapxRawVs {
     ) -> Result<()> {
         let base_br_id = self
             .inner
-            .get_branch_id(BranchName(base_branch_name.0))
+            .branch_get_id_by_name(BranchName(base_branch_name.0))
             .c(d!("base branch not found"))?;
         let base_ver_id = self
             .inner
-            .get_version_id(base_version_name)
+            .version_get_id_by_name(base_version_name)
             .c(d!("base vesion not found"))?;
         self.inner
             .branch_create_by_base_branch_version(
@@ -600,7 +677,7 @@ impl VsMgmt for MapxRawVs {
         base_branch_name: ParentBranchName,
     ) -> Result<()> {
         self.inner
-            .get_branch_id(BranchName(base_branch_name.0))
+            .branch_get_id_by_name(BranchName(base_branch_name.0))
             .c(d!("base branch not found"))
             .and_then(|base_br_id| {
                 self.inner
@@ -625,11 +702,11 @@ impl VsMgmt for MapxRawVs {
     ) -> Result<()> {
         let base_br_id = self
             .inner
-            .get_branch_id(BranchName(base_branch_name.0))
+            .branch_get_id_by_name(BranchName(base_branch_name.0))
             .c(d!("base branch not found"))?;
         let base_ver_id = self
             .inner
-            .get_version_id(base_version_name)
+            .version_get_id_by_name(base_version_name)
             .c(d!("base vesion not found"))?;
         self.inner
             .branch_create_by_base_branch_version_without_new_version(
@@ -644,7 +721,7 @@ impl VsMgmt for MapxRawVs {
     #[inline(always)]
     fn branch_exists(&self, branch_name: BranchName) -> bool {
         self.inner
-            .get_branch_id(branch_name)
+            .branch_get_id_by_name(branch_name)
             .map(|id| {
                 assert!(self.inner.branch_exists(id));
                 true
@@ -656,7 +733,7 @@ impl VsMgmt for MapxRawVs {
     #[inline(always)]
     fn branch_has_versions(&self, branch_name: BranchName) -> bool {
         self.inner
-            .get_branch_id(branch_name)
+            .branch_get_id_by_name(branch_name)
             .map(|id| self.inner.branch_has_versions(id))
             .unwrap_or(false)
     }
@@ -670,7 +747,7 @@ impl VsMgmt for MapxRawVs {
     /// and should not do any tracing.
     #[inline(always)]
     fn branch_remove(&self, branch_name: BranchName) -> Result<()> {
-        if let Some(branch_id) = self.inner.get_branch_id(branch_name) {
+        if let Some(branch_id) = self.inner.branch_get_id_by_name(branch_name) {
             self.inner.branch_remove(branch_id).c(d!())
         } else {
             Err(eg!("branch not found"))
@@ -687,7 +764,7 @@ impl VsMgmt for MapxRawVs {
     #[inline(always)]
     fn branch_truncate(&self, branch_name: BranchName) -> Result<()> {
         self.inner
-            .get_branch_id(branch_name)
+            .branch_get_id_by_name(branch_name)
             .c(d!("branch not found"))
             .and_then(|br_id| self.inner.branch_truncate(br_id).c(d!()))
     }
@@ -706,11 +783,11 @@ impl VsMgmt for MapxRawVs {
         last_version_name: VersionName,
     ) -> Result<()> {
         self.inner
-            .get_branch_id(branch_name)
+            .branch_get_id_by_name(branch_name)
             .c(d!("branch not found"))
             .and_then(|br_id| {
                 self.inner
-                    .get_version_id(last_version_name)
+                    .version_get_id_by_name(last_version_name)
                     .c(d!("version not found"))
                     .and_then(|last_ver_id| {
                         self.inner.branch_truncate_to(br_id, last_ver_id).c(d!())
@@ -728,7 +805,7 @@ impl VsMgmt for MapxRawVs {
     #[inline(always)]
     fn branch_pop_version(&self, branch_name: BranchName) -> Result<()> {
         self.inner
-            .get_branch_id(branch_name)
+            .branch_get_id_by_name(branch_name)
             .c(d!("branch not found"))
             .and_then(|id| self.inner.branch_pop_version(id).c(d!()))
     }
@@ -741,12 +818,12 @@ impl VsMgmt for MapxRawVs {
         target_branch_name: BranchName,
     ) -> Result<()> {
         self.inner
-            .get_branch_id(branch_name)
+            .branch_get_id_by_name(branch_name)
             .c(d!("branch not found"))
             .and_then(|brid| {
                 let target_brid = self
                     .inner
-                    .get_branch_id(target_branch_name)
+                    .branch_get_id_by_name(target_branch_name)
                     .c(d!("target branch not found"))?;
                 self.inner.branch_merge_to(brid, target_brid).c(d!())
             })
@@ -766,12 +843,12 @@ impl VsMgmt for MapxRawVs {
         target_branch_name: BranchName,
     ) -> Result<()> {
         self.inner
-            .get_branch_id(branch_name)
+            .branch_get_id_by_name(branch_name)
             .c(d!("branch not found"))
             .and_then(|brid| {
                 let target_brid = self
                     .inner
-                    .get_branch_id(target_branch_name)
+                    .branch_get_id_by_name(target_branch_name)
                     .c(d!("target branch not found"))?;
                 self.inner.branch_merge_to_force(brid, target_brid).c(d!())
             })
@@ -782,11 +859,38 @@ impl VsMgmt for MapxRawVs {
     #[inline(always)]
     fn branch_set_default(&mut self, branch_name: BranchName) -> Result<()> {
         self.inner
-            .get_branch_id(branch_name)
+            .branch_get_id_by_name(branch_name)
             .c(d!("branch not found"))
             .and_then(|brid| self.inner.branch_set_default(brid).c(d!()))
     }
 
+    #[inline(always)]
+    fn branch_is_empty(&self, branch_name: BranchName) -> Result<bool> {
+        self.inner
+            .branch_get_id_by_name(branch_name)
+            .c(d!("branch not found"))
+            .and_then(|brid| self.inner.branch_is_empty(brid).c(d!()))
+    }
+
+    #[inline(always)]
+    fn branch_list(&self) -> Vec<BranchNameOwned> {
+        self.inner.branch_list()
+    }
+
+    #[inline(always)]
+    fn branch_get_default(&self) -> BranchNameOwned {
+        self.inner.branch_get_default_name()
+    }
+
+    #[inline(always)]
+    unsafe fn branch_swap(
+        &mut self,
+        branch_1: BranchName,
+        branch_2: BranchName,
+    ) -> Result<()> {
+        self.inner.branch_swap(branch_1.0, branch_2.0).c(d!())
+    }
+
     /// Clean outdated versions out of the default reserved number.
     #[inline(always)]
     fn prune(&self, reserved_ver_num: Option<usize>) -> Result<()> {
